;############################################################### STOPWATCH.ASL
; Purpose.....: Stop watch with lap and reset. Uses CTC timer 0 on IOM
;		as time source. Due to the mystic clock frequency the watch
;		is not very accurate, ~1s in 6 minutes (the original clock
;		on the IOM-ROM had one second per minute).
; Target......: MPF-1P
; Assembler...: ASL V1.42 Beta
; Command Line: ASL -listline-prefix %04a -L -U -olist <name>.prn <name>.asl
; PsPadParser.: > > > %F(%L):%C: error:
;#############################################################################

;********************************************************************* history
;=========================================================== V01.00:01.03.2024
; Created
;=========================================================== V01.10:07.04.2024
; Adapted to library 24-04-07, exit code
;=========================================================== V01.11:08.04.2024
; Moved variables to MPF RAM to get continuous RAM from D800 to FD00
;=========================================================== V01.11:28.08.2024
; Changed to ASL assembler, code remains the same
;***************************************************************** end history

;*********************************************************************** notes
; Operating
;  When started, 00:00:00 ist shown, interrupts are disabled
;  SPACE toggles interrupt enable, starting and stopping the clock
;  'R' resets all counters to zero
;  SHIFT & CONTROL exits
;******************************************************************* end notes

;***************************************************************** definitions
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++ assembler directives
		CPU	Z80		; use Z80 CPU
		PAGE	0		; no formfeeds in listing file
		
;+++++++++++++++++++++++++++++++++++ hardware, monitor and library definitions
		LISTING	OFF
		INCLUDE	../LIBRARY/MPF1P.DEF
		LISTING	ON
		
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ build addresses
; The code contains jumps over IO addresses. It must be assembled at code
; addresses xx00, otherwise it will crash due to improper I/O decoding.
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wSWCS:		EQU	0B500H		; code: B500=IOM ROM, xx00 otherwise
wSWDS:		EQU	0FD00H		; where data goes

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ variables
wSWVIS:		EQU	wSWDS		; vars init start mark
vSWCI0:		EQU	wSWVIS		; CTC-0 ints counters
vSWHR:		EQU	vSWCI0 + 2	; hours counter
vSWMIN:		EQU	vSWHR  + 1	; minutes counter
vSWSEC:		EQU	vSWMIN + 1	; seconds counter
vSWSEH:		EQU	vSWSEC + 1	; 100mS counter
vSWICC:		EQU	vSWSEH + 1	; interrupt callback output counter
vSWF:		EQU	vSWICC + 1	; various flags:
iSWHMS:		EQU	0		;  bit0: 100mS elapsed
iSWIR:		EQU	1		;  bit1: ints stopped
nSWIR:		EQU	02H		;        flag value
wSWVIE:		EQU	vSWF   + 1	; vars init end mark
bSWVIZ:		EQU	wSWVIE - wSWVIS	; variables init size

;================================================================ text buffers
wSWTA:		EQU	vSWF + 1	; RAM text copy destination
vSWTI:		EQU	wSWTA		; timer display
wSWVE:		EQU	vSWTI + dbMCPL	; variables end address

;======================================== check overlap with library variables
		IF	wSWVE > dwLVST && MomPass = 2
		WARNING	"Variables overlap with library:\{wSWVE} >> \{dwLVST}"
		ENDIF
;************************************************************* end definitions

;************************************************************** implementation
		ASEG
		ORG	wSWCS

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ main program
;======================================================================== SW()
; Purpose.....: Main program
; Labels......: S)top W)atch
;=============================================================================
SW:		CALL	dfLLS		; setup library variables

		LD	HL,wSWVIS	; init start address
		LD	BC,bSWVIZ	; init size
		XOR	A		; init value
		CALL	dfLHMF		; fill memory

		LD	HL,wSWTO	; text constants (might be in ROM)
		LD	DE,wSWTA	; RAM address
		LD	BC,bSWTZ	; size of texts
		LDIR			; copy text buffers to RAM

		LD	IY,vSWF		; IY= flags address

		LD	IX,tSWSO	; IX= sign-on message
		LD	B,040H		; show 1.5 seconds
		CALL	dfLDT
		LD	IX,vSWTI	; IX= timer message

		LD	A,2		; use IM2
		CALL	dfLIS		; place CPU in interrupt mode

;------------------------------------------------------------- setup CTC timer
		LD	D,dbIC0		; timer 0
		LD	E,dbCI2		; timer mode, interrupt, prescaler 256
		LD	B,dbCO10	; 10mS interval
		LD	HL,SWC0CB	; timer 0 callback address
		SCF			; start timer
		CALL	dfLCS		; setup timer 0

;------------------------------------------- setup PIO-A for interrupt display
		LD	D,dbIPA		; channel A
		LD	E,dbPMO		; mode 0, output
		LD	B,dbPID		; disable interrupts
		SCF			; set it up
		CALL	dfLPS		; setup PIO channel A

;----------------------------------------------------------- display the timer
SWL:		CALL	dfLDK
		JR	C,SWX		; shift&control, exit
		JR	Z,SWTU		; no key, update timers

;---------------------------------------------------------- process input keys
		CP	'R'		; R)eset?
		CALL	Z,SWRSC		; yes, reset counters

		CP	dbSP		; space? (start/stop interrupt)
		CALL	Z,SWSS		; yes, toggle start/stop

;-------------------------------------------------------- update timer display
SWTU:		BIT	iSWHMS,(IY)	; 100ms over?
		JR	Z,SWL		; no, back to loop

		JR	ISIOM1		;-------------------------------------
		DS	12H		; $$$ IOM-MPF: skip I/O xx6x
ISIOM1:		EQU	$		;-------------------------------------

		RES	iSWHMS,(IY)	; reset 100mS flag
		CALL	SWCU		; update counters
		JR	SWL		; and back to loop

;------------------------------------------------------------------- terminate
SWX:		DI			; disable interrupts

		XOR	A
		LD	D,dbIC0
		CALL	dfLCS		; stop timer 0

		XOR	A
		LD	D,dbIPA
		CALL	dfLPS		; reset PIO-A

		JP	dwMRET		; monitor warm start
;==================================================================== eoM SW()
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end main program

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ subroutines
;====================================================================== SWSS()
; Purpose.....: Toggle interrupts on/off, if off, show full 10mS counter
; Registers...:	AF destroyed
; Labels......: S)topW)atch S)tart S)top
;=============================================================================
SWSS:		PUSH	HL
		PUSH	DE
		PUSH	BC

		LD	HL,vSWTI+bSWTIU	; position of 10mS display

		LD	A,nSWIR		; A= ints running flag value
		XOR	(IY)		; invert it
		LD	(IY),A		; and write back

		BIT	iSWIR,(IY)	; running?
		JR	Z,SWDI		; yes, disable interrupts
		INC	HL		; 2nd digit
		LD	(HL),'0'	; clear to zero
		EI			; enable ints
		JR	SWSSX		; and exit

SWDI:		DI			; stop request, disable ints
		LD	A,(vSWCI0+1)	; A= 10mS counter for running second
		LD	D,0
		LD	E,A		; DE= 10mS counter
		LD	C,2		; 2 digits
		SCF			; leading zeros
		CALL	dfLOBD		; convert to decimal

SWSSX:		POP	BC
		POP	DE
		POP	HL
		RET
;================================================================== eoF SWSS()

;====================================================================== SWCU()
; Purpose.....: Update display counters, called every 100mS
; Registers...: All saved
; Labels......: S)topW)atch C)ounters U)pdate
;=============================================================================
SWCU:		PUSH	HL
		PUSH	DE
		PUSH	BC

		LD	D,0		; hibyte of convert value = 0
		LD	C,2		; number of digits to convert

		LD	HL,vSWSEH	; HL= address of 100mS counter
		INC	(HL)
		LD	A,(vSWSEH)	; A= 100mS counter
		CP	10		; one second over?
		JR	NZ,SWCUU	; no, just update 100mS display

		DEC	HL		; yes, HL= address of seconds counter
		INC	(HL)		; bump seconds

		JR	ISPRT1		;-------------------------------------
		DS	02H		; $$$ IOM-PRT: skip I/O xxCA,xxCB
ISPRT1:		EQU	$		;-------------------------------------

		LD	A,(vSWSEC)	; A= seconds counter
		CP	60		; one minute over?
		JR	NZ,SWCUS	; no, update seconds and 100mS

		DEC	HL		; yes, HL= address of minutes counter
		INC	(HL)		; bump minutes
		LD	A,(vSWMIN)	; A= minutes counter
		CP	60		; one hour over?
		JR	NZ,SWCUM	; no, update min,secs,100mS

		DEC	HL		; yes, HL= address of hours counter
		INC	(HL)		; bump hours
		LD	A,(vSWHR)	; A= hours counter
		CP	24		; one day over?
		JR	NZ,SWCUH	; no, update hrs,mins,secs,100mS
		XOR	A
		LD	(vSWHR),A	; yes, clear hours counter

SWCUH:		LD	HL,vSWTI+bSWTIH	; HL= position of hours display
		LD	A,(vSWHR)
		LD	E,A		; DE= value to convert
		SCF			; leading zeros
		CALL	dfLOBD		; convert binary to decimal
		XOR	A
		LD	(vSWMIN),A	; hours changed, clear minutes

SWCUM:		LD	HL,vSWTI+bSWTIM	; HL= position of minutes display
		LD	A,(vSWMIN)
		LD	E,A		; DE= value to convert
		SCF			; leading zeros
		CALL	dfLOBD		; convert binary to decimal
		XOR	A
		LD	(vSWSEC),A	; minutes changed, clear seconds

SWCUS:		LD	HL,vSWTI+bSWTIS	; HL= position of seconds display
		LD	A,(vSWSEC)
		LD	E,A		; DE= value to convert
		SCF			; leading zeros
		CALL	dfLOBD		; convert binary to decimal
		XOR	A
		LD	(vSWSEH),A	; seconds changed, clear 10mS

SWCUU:		LD	HL,vSWTI+bSWTIU	; HL= position of mS display
		LD	A,(vSWSEH)
		LD	C,1		; only one digit
		LD	E,A		; DE= value to convert
		SCF			; leading zeros
		CALL	dfLOBD		; convert binary to decimal

		POP	BC
		POP	DE
		POP	HL
		RET
;================================================================== eoF SWCU()

;===================================================================== SWRSC()
; Purpose.....: Reset all counters to zero
; Registers...: All saved
; Labels......: S)topW)atch R)eS)et C)ounters
;=============================================================================
SWRSC:		PUSH	HL
		PUSH	BC

		CALL	dfLID		; disable interrupts

		LD	HL,vSWCI0	; counters start address
		LD	B,7		; number of counters to clear

SWRSCL:		LD	(HL),0		; clear all counters, including
		INC	HL		; the interrupt one
		DJNZ	SWRSCL

		RES	iSWHMS,(IY)	; also reset 100mS flag

		LD	HL,tSWTI	; re-init display text
		LD	DE,vSWTI	; RAM address
		LD	BC,20		; size of text
		LDIR			; copy text buffer

		JR	NC,SWRSCX	; ints were not enabled
		EI			; they were enabled, re-enable them

SWRSCX:		POP	BC
		POP	HL
		RET
;================================================================= eoF SWRSC()
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end subroutines

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++ interrupt callbacks
;==================================================================== LTC0CB()
; Purpose.....: CTC timer 0 interrupt callback. Runs in 10mS interrupt context
; Registers...: All saved
; Labels......: S)topW)atch C)TC timer 0) C)allB)ack
;=============================================================================
SWC0CB:		PUSH	AF
		PUSH	HL

		LD	A,(vSWICC)	; get callback counter
		INC	A		; bump it
		LD	(vSWICC),A	; and write back
		OUT	(dpIPAD),A	; output to PIO-A

		LD	HL,(vSWCI0)	; get ints counters
		INC	L		; bump 100mS counter
		INC	H		; bump one second counter

		LD	A,9
		CP	L		; 100mS over?
		JR	NC,SWC0CX	; no, exit
		LD	L,0		; yes, reset 100mS counter

		LD	A,(vSWF)	; set 100mS elapsed flag
		SET	iSWHMS,A
		LD	(vSWF),A

		LD	A,99
		CP	H		; one second over?
		JR	NC,SWC0CX	; no, exit
		LD	H,0		; yes, reset one second counter

SWC0CX:		LD	(vSWCI0),HL
		POP	HL
		POP	AF
		RET
;================================================================ eoF SWC0CB()
;+++++++++++++++++++++++++++++++++++++++++++++++++++++ end interrupt callbacks

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ text constants
tSWSO:		DB	'    SW V24-08-28    '

;--------------------------------------------------------- texts copied to RAM
wSWTO:		EQU	$               ; ROM text copy source
tSWTI:		DB	'    00:00:00-00     '
bSWTIH:		EQU	4               ; position hours
bSWTIM:		EQU	7               ;          minutes
bSWTIS:		EQU	10              ;          seconds
bSWTIU:		EQU	13              ;          10mS
wSWTE:		EQU	$               ; end of variable text
bSWTZ:		EQU	wSWTE - wSWTO	; size of variable text to copy to RAM
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end text constants

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ size info
wSWCSE:		EQU	$

		IF	MomPass = 2
		MESSAGE "Code from \{wSWCS} to \{wSWCSE} Size \{wSWCSE-wSWCS}"
		MESSAGE "Data from \{wSWDS} to \{wSWVE} Size \{wSWVE-wSWDS}"
		MESSAGE "     remaining \{dwLVST-wSWVE}"
		ENDIF
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end size info

;########################################################### EOF STOPWATCH.ASL
		END
